@@ -83,7 +83,7 @@ pub(crate) fn run(args: flags::InitDatabase) -> Result<()> {
             continue;
         }
         let line = line?;
-        match parse_line(line_num, delimiter, &line, args.keep_word_freq) {
+        match parse_line(line_num, delimiter, &line, args.keep_word_freq, args.fix) {
             Ok((syllables, phrase, freq)) => builder.insert(&syllables, (phrase, freq).into())?,
             Err(error) => errors.push(error),
         };
@@ -92,6 +92,9 @@ pub(crate) fn run(args: flags::InitDatabase) -> Result<()> {
         for err in errors {
             eprintln!("{}", err);
         }
+        if !args.fix {
+            eprintln!("Hint: Use --fix to automatically fix common errors");
+        }
         if !args.skip_invalid {
             std::process::exit(1)
         }
@@ -122,10 +125,11 @@ fn parse_line(
     delimiter: char,
     line: &str,
     keep_word_freq: bool,
+    fix: bool,
 ) -> Result<(Vec<Syllable>, &str, u32)> {
     let phrase = line
         .split(delimiter)
-        .next()
+        .find(|s| !s.is_empty())
         .ok_or(parse_error(line_num, line))?
         .trim_matches('"');
 
@@ -133,6 +137,7 @@ fn parse_line(
         1 if !keep_word_freq => 0,
         _ => line
             .split(delimiter)
+            .filter(|s| !s.is_empty())
             .nth(1)
             .ok_or(parse_error(line_num, line))?
             .trim_matches('"')
@@ -142,33 +147,50 @@ fn parse_line(
     };
 
     let mut syllables = vec![];
-    for syllables_field in line.splitn(3, delimiter).skip(2) {
-        let syllables_field = syllables_field.trim_matches('"');
-        for syllable_str in syllables_field.split_whitespace() {
-            if syllable_str.is_empty() {
-                continue;
-            }
-            let mut syllable_builder = Syllable::builder();
-            if syllable_str.starts_with('#') {
-                break;
-            }
-            for c in syllable_str.chars() {
-                syllable_builder = syllable_builder
-                    .insert(
-                        Bopomofo::try_from(c)
-                            .context("parsing bopomofo")
-                            .parse_error(line_num, line)?,
-                    )
-                    .with_context(|| format!("Parsing syllables {}", syllable_str))
-                    .parse_error(line_num, line)?;
-            }
-            syllables.push(syllable_builder.build());
+
+    for syllable_str in line
+        .split(|c: char| c == ',' || c.is_whitespace())
+        .filter(|s| !s.is_empty())
+        .skip(2)
+    // skip phrase and freq
+    {
+        let syllable_str = syllable_str.trim_matches('"');
+        if syllable_str.is_empty() {
+            continue;
         }
+        let mut syllable_builder = Syllable::builder();
+        if syllable_str.starts_with('#') {
+            break;
+        }
+        for c in syllable_str.chars() {
+            let c = if fix {
+                fix_common_syllable_errors(c)
+            } else {
+                c
+            };
+            syllable_builder = syllable_builder
+                .insert(
+                    Bopomofo::try_from(c)
+                        .context("parsing bopomofo")
+                        .parse_error(line_num, line)?,
+                )
+                .with_context(|| format!("Parsing syllables {}", syllable_str))
+                .parse_error(line_num, line)?;
+        }
+        syllables.push(syllable_builder.build());
     }
 
     Ok((syllables, phrase, freq))
 }
 
+fn fix_common_syllable_errors(c: char) -> char {
+    match c {
+        '一' => 'ㄧ',
+        '丫' => 'ㄚ',
+        _ => c,
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use chewing::syl;
@@ -179,7 +201,7 @@ mod tests {
     #[test]
     fn parse_ssv() {
         let line = "鑰匙 668 ㄧㄠˋ ㄔˊ # not official";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false, false) {
             assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
             assert_eq!("鑰匙", phrase);
             assert_eq!(668, freq);
@@ -188,10 +210,42 @@ mod tests {
         }
     }
 
+    #[test]
+    fn parse_ssv_multiple_whitespace() {
+        let line = "鑰匙     668 ㄧㄠˋ ㄔˊ # not official";
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false, false) {
+            assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
+            assert_eq!("鑰匙", phrase);
+            assert_eq!(668, freq);
+        } else {
+            panic!()
+        }
+    }
+
+    #[test]
+    fn parse_ssv_syllable_errors() {
+        let line = "地永天長 50 ㄉ一ˋ ㄩㄥˇ ㄊ一ㄢ ㄔ丫ˊ";
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ' ', &line, false, true) {
+            assert_eq!(
+                syllables,
+                vec![
+                    syl![D, I, TONE4],
+                    syl![IU, ENG, TONE3],
+                    syl![T, I, AN],
+                    syl![CH, A, TONE2]
+                ]
+            );
+            assert_eq!("地永天長", phrase);
+            assert_eq!(50, freq);
+        } else {
+            panic!()
+        }
+    }
+
     #[test]
     fn parse_csv() {
         let line = "鑰匙,668,ㄧㄠˋ ㄔˊ # not official";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false, false) {
             assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
             assert_eq!("鑰匙", phrase);
             assert_eq!(668, freq);
@@ -203,7 +257,7 @@ mod tests {
     #[test]
     fn parse_csv_quoted() {
         let line = "\"鑰匙\",668,\"ㄧㄠˋ ㄔˊ # not official\"";
-        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false) {
+        if let Ok((syllables, phrase, freq)) = parse_line(0, ',', &line, false, false) {
             assert_eq!(syllables, vec![syl![I, AU, TONE4], syl![CH, TONE2]]);
             assert_eq!("鑰匙", phrase);
             assert_eq!(668, freq);
